@@ -1,5 +1,7 @@
 #!/usr/bin/perl -w
 #
+#    vi: set ts=2 sw=2 noai ic showmode showmatch:  
+#
 #    Copyright (C) 1998, Dj Padzensky <djpadz@padz.net>
 #    Copyright (C) 1998, 1999 Linas Vepstas <linas@linas.org>
 #    Copyright (C) 2000, Yannick LE NY <y-le-ny@ifrance.com>
@@ -46,13 +48,16 @@ use vars qw/@ISA @EXPORT @EXPORT_OK @EXPORT_TAGS
             $TIMEOUT @MODULES %MODULES %METHODS $AUTOLOAD
             @CURRENCY_RATES_MODULES $USE_EXPERIMENTAL_UA/;
 
-our $VERSION = '1.54'; # VERSION
+our $VERSION = '1.64'; # VERSION
 
 @CURRENCY_RATES_MODULES = qw/
     AlphaVantage
+    CurrencyFreaks
     ECB
+    FinanceAPI
     Fixer
     OpenExchange
+    YahooJSON
 /;
 
 @MODULES = qw/
@@ -62,27 +67,29 @@ our $VERSION = '1.54'; # VERSION
     AlphaVantage
     BSEIndia
     Bloomberg
+    BorsaItaliana
     Bourso
+    BVB
     CSE
-    Cdnfundlibrary
     Comdirect
+    Consorsbank
     Currencies
     DWS
     Deka
-    FTfunds
-    Fidelity
+    FinanceAPI
     Finanzpartner
     Fondsweb
     Fool
-    Fundata
+    FTfunds
     GoldMoney
+    GoogleWeb
     HU
-    IEXCloud
     IndiaMutual
-    MStaruk
+    MarketWatch
     MorningstarAU
     MorningstarCH
     MorningstarJP
+    MorningstarUK
     NSEIndia
     NZX
     OnVista
@@ -90,23 +97,26 @@ our $VERSION = '1.54'; # VERSION
     SEB
     SIX
     Sinvestor
+    StockData
+    Stooq
     TesouroDireto
     Tiaacref
     TMX
     Tradegate
-    Tradeville
     TreasuryDirect
     Troweprice
     TSP
+    TwelveData
     Union
     XETRA
     YahooJSON
+    YahooWeb
     ZA
 /;
 
 @ISA    = qw/Exporter/;
 @EXPORT = ();
-@EXPORT_OK = qw/fidelity troweprice asx tiaacref
+@EXPORT_OK = qw/troweprice asx tiaacref
                 currency_lookup/;
 @EXPORT_TAGS = ( all => [@EXPORT_OK]);
 
@@ -251,7 +261,9 @@ sub _load_modules {
 
         foreach my $method (keys %methodhash) {
           push (@{$METHODS{$method}},
-              { function => $methodhash{$method},
+              { name => $module,
+              modpath => $modpath,
+              function => $methodhash{$method},
               labels   => $labelhash{$method},
               currency_fields => \@currency_fields});
         }
@@ -315,6 +327,48 @@ sub get_methods {
   return(wantarray ? keys %METHODS : [keys %METHODS]);
 }
 
+# return hash:
+#
+#  quote_methods => hash of
+#      method_name => array of module names
+#  quote_modules => hash of
+#      module_name => array of parameters
+#  currency_modules => hash of
+#      module_name =>  array of parameters
+#
+# { 
+#    'quote_methods' => {'group' => ['module', 'module'], ...},
+#    'quote_modules' => {'abc' => ['API_KEY'], ...},
+#    'currency_modules' => {'xyz' => [], 'lmn' => ['USER_NAME', 'API_KEY']},
+# } 
+
+sub get_features {
+  # Create a dummy object to ensure METHODS is populated
+  my $t = Finance::Quote->new(currency_rates => {order => \@CURRENCY_RATES_MODULES});
+  my $baseclass = ref $t;
+
+  my %feature = (
+    'quote_methods' => {map {$_, [map {$_->{name}} @{$METHODS{$_}}]} keys %METHODS},
+    'quote_modules' => {map {$_, []} @MODULES},
+    'currency_modules' => {map {$_, []} @CURRENCY_RATES_MODULES},
+  );
+
+  my %mods = ('quote_modules' => $baseclass,
+              'currency_modules' => "${baseclass}::CurrencyRates");
+
+  while (my ($field, $base) = each %mods) {
+    foreach my $name (keys %{$feature{$field}}) {
+      my $modpath = "${base}::${name}";
+
+      if ($modpath->can("parameters")) {
+        push (@{$feature{$field}->{$name}}, $modpath->parameters());
+      }
+    }
+  }
+
+  return %feature;
+}
+
 # =======================================================================
 # new (public class method)
 #
@@ -357,11 +411,24 @@ sub new {
   # 1. Add a default value for $this->{object-name}
   # 2. Add the 'user-visible-name' => [type, object-name] to %named_parameter
 
+  # Check for FQ_CURRENCY - preferred currency module
+  # Set to AlphaVantage if not set or not in @CURRENCY_RATES_MODULES
+  my $CURRENCY_MODULE;
+  if (!$ENV{FQ_CURRENCY}) {
+    $CURRENCY_MODULE='AlphaVantage';
+  } else {
+    if ( grep( /^$ENV{FQ_CURRENCY}$/, @CURRENCY_RATES_MODULES ) ) {
+      $CURRENCY_MODULE=$ENV{FQ_CURRENCY}
+    } else {
+      $CURRENCY_MODULE='AlphaVantage';
+    }
+  }
+
   # Default values
   $this->{FAILOVER}       = 1;
   $this->{REQUIRED}       = [];
   $this->{TIMEOUT}        = $TIMEOUT if defined($TIMEOUT);
-  $this->{currency_rates} = {order => ['AlphaVantage']};
+  $this->{currency_rates} = {order => [$CURRENCY_MODULE]};
 
   # Sort out arguments
   my %named_parameter = (timeout         => ['', 'TIMEOUT'],
@@ -540,6 +607,15 @@ sub fetch {
     return;
   }
 
+  # Temporary Counting - not concerned about return code
+  # my $COUNT_URL =
+  #   'http://www.panix.com/~hd-fxsts/finance-quote.html?' . $method;
+  # my $count_ua = LWP::UserAgent->new(timeout => 10);
+  # my $count_response = $count_ua->head($COUNT_URL);
+
+  # ### COUNT_URL: $COUNT_URL
+  # ### Code: $count_response->code
+
   # Failover code.  This steps through all available methods while
   # we still have failed stocks to look-up.  This loop only
   # runs a single time unless FAILOVER is defined.
@@ -710,7 +786,7 @@ sub store_date
       $year += 2000 if $year < 100;
       $year_specified = 1;
 
-      ### format: printf "usdate %s -> Day %d, Month %s, Year %d\n", $piecesref->{usdate}, $day, $month, $year
+      ### format: printf STDERR "usdate %s -> Day %d, Month %s, Year %d\n", $piecesref->{usdate}, $day, $month, $year
     }
 
     if ((defined $piecesref->{eurodate}) && ($piecesref->{eurodate})) {
@@ -718,7 +794,7 @@ sub store_date
       $year += 2000 if $year < 100;
       $year_specified = 1;
 
-      ### format: printf "eurodate %s -> Day %d, Month %s, Year %d\n", $piecesref->{eurodate}, $day, $month, $year
+      ### format: printf STDERR "eurodate %s -> Day %d, Month %s, Year %d\n", $piecesref->{eurodate}, $day, $month, $year
     }
 
     if (defined ($piecesref->{year})) {
@@ -744,7 +820,7 @@ sub store_date
     $month = $mnames{lc(substr($month,0,3))} if ($month =~ /\D/);
     $year-- if (($year_specified == 0) && ($this_month < $month));
 
-    ### format: printf "Final Year-Month-Day -> %04d-%02d-%02d\n", $year, $month, $day
+    ### format: printf STDERR "Final Year-Month-Day -> %04d-%02d-%02d\n", $year, $month, $day
 
     $inforef->{$symbol, "date"} =  sprintf "%02d/%02d/%04d", $month, $day, $year;
     $inforef->{$symbol, "isodate"} = sprintf "%04d-%02d-%02d", $year, $month, $day;
@@ -1052,6 +1128,8 @@ sub user_agent {
 
 __END__
 
+=for comment README.md generated from lib/Finance/Quote.pm
+
 =head1 NAME
 
 Finance::Quote - Get stock and mutual fund quotes from various exchanges
@@ -1191,11 +1269,9 @@ methods.
 
     my $q = Finance::Quote->new()
     my $q = Finance::Quote->new('-defaults')
-    my $q = Finance::Quote->new('AEX', 'Fool')
     my $q = Finance::Quote->new(timeout => 30)
     my $q = Finance::Quote->new('YahooJSON', fetch_currency => 'EUR')
     my $q = Finance::Quote->new('alphavantage' => {API_KEY => '...'})
-    my $q = Finance::Quote->new('IEXCloud', 'iexcloud' => {API_KEY => '...'});
     my $q = Finance::Quote->new(currency_rates => {order => ['ECB', 'Fixer'], 'fixer' => {API_KEY => '...'}});
 
 Finance::Quote modules access a wide range of sources to provide quotes.  A
@@ -1261,8 +1337,11 @@ Finance::Quote::CurrencyRates::AlphaVantage is used for currency conversion.
 This end point requires an API key, which can either be set in the environment
 or included in the configuration hash. To specify a different primary currency
 conversion method or configure fallback methods, include the 'order' key, which
-points to an array of Finance::Quote::CurrencyRates module names. See the
-documentation for the individual Finance::Quote::CurrencyRates to learn more. 
+points to an array of Finance::Quote::CurrencyRates module names.
+Setting the environment variable FQ_CURRENCY will change the default 
+endpoint used for currency conversion.
+See the documentation for the individual Finance::Quote::CurrencyRates
+modules to learn more. 
 
 =head2 get_default_currency_fields
 
@@ -1294,6 +1373,37 @@ C<set_default_timeout> sets the Finance::Quote default timeout to a new value.
 C<get_methods> returns the list of methods that can be passed to C<new> when
 creating a quoter object and as the first argument to C<fetch>.
 
+=head2 get_features
+
+    my %features = Finance::Quote::get_features();
+
+C<get_features> returns a hash with three keys: quote_methods, quote_modules, and currency_modules.
+
+    $features{quote_methods} is a hash with key/value pairs of method_name => [array of module names]
+    $features{quote_modules} is a hash with key/value pairs of module_name => [array of parameter names]
+    $features{currency_modules} is a hash with key/value pairs of currency_module_name => [array of paramater names]
+
+Parameter names are values that the module needs to function, such as API_KEY. Most
+modules will have an empty list.  Modules with a parameter are configured when creating
+the Finance::Quote by passing the argument
+
+   'module_name_in_lower_case' => {paramter => value}
+
+to Finance::Quote->new().
+
+The keys of the $features{currency_modules} hash are the names of currency
+modules that can be used for currency conversion and the order in which the
+modules are used is controlled by the argument
+
+    currency_rates => {order => [subset of $features{currency_modules}]} 
+
+to Finance::Quote->new().  By default, AlphaVantage in used for 
+currency conversion, "order" can be set to use other currency modules.
+The currency module used can also be changed by setting the FQ_CURRENCY
+environment variable. Please note that some of the alternate currency
+modules require API keys.
+
+
 =head1 PUBLIC OBJECT METHODS
 
 =head2 B_to_billions
@@ -1438,8 +1548,8 @@ second argument.  In the above example, C<value> is C<'10.23'>.
 C<currency> converts a value with a currency code suffix to another currency
 using the current exchange rate as determined by the
 Finance::Quote::CurrencyRates method or methods configured for the quoter $q.
-When called as a class method, only Finance::Quote::AlphaVantage is used, which
-requires an API key. See the introduction for information on configuring
+When called as a class method, Finance::Quote::AlphaVantage is the default,
+which requires an API key. See the introduction for information on configuring
 currency rate conversions and see Finance::Quote::CurrencyRates::AlphaVantage
 for information about the API key.
 
@@ -1527,7 +1637,6 @@ information when compared to references
  Copyright 2000 Volker Stuerzl (DWS)
  Copyright 2001 Rob Sessink (AEX support)
  Copyright 2001 Leigh Wedding (ASX updates)
- Copyright 2001 Tobias Vancura (Fool support)
  Copyright 2001 James Treacy (TD Waterhouse support)
  Copyright 2008 Erik Colson (isoTime)
 
@@ -1544,19 +1653,32 @@ Please refer to the sub-modules for further information.
 
 =head1 AUTHORS
 
-  Dj Padzensky <djpadz@padz.net>, PadzNet, Inc.
-  Linas Vepstas <linas@linas.org>
-  Yannick LE NY <y-le-ny@ifrance.com>
-  Paul Fenwick <pjf@cpan.org>
-  Brent Neal <brentn@users.sourceforge.net>
-  Volker Stuerzl <volker.stuerzl@gmx.de>
-  Keith Refson <Keith.Refson#earth.ox.ac.uk>
-  Rob Sessink <rob_ses@users.sourceforge.net>
-  Leigh Wedding <leigh.wedding@telstra.com>
-  Tobias Vancura <tvancura@altavista.net>
-  James Treacy <treacy@debian.org>
-  Bradley Dean <bjdean@bjdean.id.au>
+The following list are those contributors with 5 or more commits in the
+GitHub repository.
+Please see the Git log for all the authors who have helped with
+Finance::Quote.
+
   Erik Colson <eco@ecocode.net>
+  Bruce Schuck <bschuck@asgard-systems.com>
+  Paul Fenwick <pjf@perltraining.com.au>
+  Vincent Lucarelli <vincent.lucarelli@gmail.com>
+  David Hampton <hampton-sf@rainbolthampton.net>
+  jvolkening <jdv@base2bio.com>
+  Pawel Konieczny <konieczp@zonnet.nl>
+  Mike Alexander <mta@umich.edu>
+  AndreJ <Andre-J@users.noreply.github.com>
+  Vinay S Shastry <vinayshastry@gmail.com>
+  Bradley Dean <bjdean@bjdean.id.au>
+  Brent Neal <brent@reindeergraphics.com>
+  Jacinta Richardson <jarich@perltraining.com.au>
+  e-dorigatti <emilio.dorigatti@gmail.com>
+  goodvibes2 <goodchris96@gmail.com>
+  Paul Howarth <paul@city-fan.org>
+  Sam Morris <sam@robots.org.uk>
+  Linas Vepstas <linas@linas.org>
+  Rajan Vaswani <111571283+bgr22112@users.noreply.github.com>
+  Hiranya Samarasekera <hiranyas@gmail.com>
+  Manuel Friedli <manuel@fritteli.ch>
 
 The Finance::Quote home page can be found at
 http://finance-quote.sourceforge.net/
@@ -1569,56 +1691,70 @@ http://www.gnucash.org/
 
 =head1 SEE ALSO
 
-Finance::Quote::CurrencyRates::AlphaVantage,
-Finance::Quote::CurrencyRates::ECB,
-Finance::Quote::CurrencyRates::Fixer,
-Finance::Quote::CurrencyRates::OpenExchange,
-Finance::Quote::AEX,
-Finance::Quote::ASEGR,
-Finance::Quote::ASX,
-Finance::Quote::Bloomberg,
-Finance::Quote::BSEIndia,
-Finance::Quote::Bourso,
-Finance::Quote::CSE,
-Finance::Quote::Cdnfundlibrary,
-Finance::Quote::Comdirect,
-Finance::Quote::Currencies,
-Finance::Quote::DWS,
-Finance::Quote::Deka,
-Finance::Quote::FTfunds,
-Finance::Quote::Fidelity,
-Finance::Quote::Finanzpartner,
-Finance::Quote::Fondsweb,
-Finance::Quote::Fool,
-Finance::Quote::Fundata
-Finance::Quote::GoldMoney,
-Finance::Quote::HU,
-Finance::Quote::IEXCloud,
-Finance::Quote::IndiaMutual,
-Finance::Quote::MStaruk,
-Finance::Quote::MorningstarAU,
-Finance::Quote::MorningstarCH,
-Finance::Quote::MorningstarJP,
-Finance::Quote::NSEIndia,
-Finance::Quote::NZX,
-Finance::Quote::OnVista,
-Finance::Quote::Oslobors,
-Finance::Quote::SEB,
-Finance::Quote::SIX,
-Finance::Quote::Tradeville,
-Finance::Quote::TSP,
-Finance::Quote::TMX,
-Finance::Quote::Tiaacref,
-Finance::Quote::TesouroDireto,
-Finance::Quote::TreasuryDirect,
-Finance::Quote::Troweprice,
-Finance::Quote::Union,
-Finance::Quote::YahooJSON,
-Finance::Quote::ZA
+  Finance::Quote::CurrencyRates::AlphaVantage,
+  Finance::Quote::CurrencyRates::CurrencyFreaks,
+  Finance::Quote::CurrencyRates::ECB,
+  Finance::Quote::CurrencyRates::FinanceAPI,
+  Finance::Quote::CurrencyRates::Fixer,
+  Finance::Quote::CurrencyRates::OpenExchange,
+  Finance::Quote::CurrencyRates::YahooJSON,
+  Finance::Quote::AEX,
+  Finance::Quote::ASEGR,
+  Finance::Quote::ASX,
+  Finance::Quote::Bloomberg,
+  Finance::Quote::BorsaItaliana,
+  Finance::Quote::BSEIndia,
+  Finance::Quote::Bourso,
+  Finance::Quote::BVB,
+  Finance::Quote::CSE,
+  Finance::Quote::Comdirect,
+  Finance::Quote::Consorsbank,
+  Finance::Quote::Currencies,
+  Finance::Quote::DWS,
+  Finance::Quote::Deka,
+  Finance::Quote::FinanceAPI,
+  Finance::Quote::Finanzpartner,
+  Finance::Quote::Fondsweb,
+  Finance::Quote::Fool,
+  Finance::Quote::FTfunds,
+  Finance::Quote::GoldMoney,
+  Finance::Quote::GoogleWeb,
+  Finance::Quote::HU,
+  Finance::Quote::IndiaMutual,
+  Finance::Quote::MarketWatch,
+  Finance::Quote::MorningstarAU,
+  Finance::Quote::MorningstarCH,
+  Finance::Quote::MorningstarJP,
+  Finance::Quote::MorningstarUK,
+  Finance::Quote::NSEIndia,
+  Finance::Quote::NZX,
+  Finance::Quote::OnVista,
+  Finance::Quote::Oslobors,
+  Finance::Quote::SEB,
+  Finance::Quote::SIX,
+  Finance::Quote::StockData,
+  Finance::Quote::Stooq,
+  Finance::Quote::TSP,
+  Finance::Quote::TMX,
+  Finance::Quote::Tiaacref,
+  Finance::Quote::TesouroDireto,
+  Finance::Quote::TreasuryDirect,
+  Finance::Quote::Troweprice,
+  Finance::Quote::TwelveData,
+  Finance::Quote::Union,
+  Finance::Quote::YahooJSON,
+  Finance::Quote::YahooWeb,
+  Finance::Quote::ZA
 
 You should have received the Finance::Quote hacker's guide with this package.
 Please read it if you are interested in adding extra methods to this package.
 The latest hacker's guide can also be found on GitHub at
 https://github.com/finance-quote/finance-quote/blob/master/Documentation/Hackers-Guide
 
+=head1 Donations
+
+While the contributors to Finance::Quote are happy to volunteer their time and resources on this project, donations are helpful and great way to say thank you. So far the only real cost to the team is domain registration for finance-quote.org. However, some operating systems and hardware upgrades do have a cost. Access to the latest or current environments is needed for development and testing. Upgrades to the latest MS Windows often includes a cost, and recently Apple's move to ARM/M1/M2 CPUs has stifled some contributors' ability to assist. With that in mind, you can send donations to the email address donations@finance-quote.org through PayPal. If it's for a particular contributor, please indicate in a note while submitting the donation through PayPal.
+
+Additionally, Bitcoin donations can be received at 37Bg4EMAp575j4iL9jHv1ZhDgiropU38jY.
+
 =cut
