@@ -16,6 +16,9 @@
 #    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
 #    02110-1301, USA
 
+# 2024-04-07: Commented call to sleep_before_query in get_content
+#             subroutine. AlphaVantage is not throttling API calls,
+#             but limits usage with free API key to 25 per day.
 # 2019-12-01: Added additional labels for net and p_change. Set
 #             close to previous close as returned in the JSON.
 #             Bruce Schuck (bschuck at asgard hyphen systems dot com)
@@ -26,7 +29,7 @@ use strict;
 use JSON qw( decode_json );
 use HTTP::Request::Common;
 
-our $VERSION = '1.51'; # VERSION
+our $VERSION = '1.64'; # VERSION
 
 # Alpha Vantage recommends that API call frequency does not extend far
 # beyond ~1 call per second so that they can continue to deliver
@@ -53,12 +56,13 @@ my %currencies_by_suffix = (
     '.PK'  => "USD",    # USA		Pink Sheets
     '.X'   => "USD",    # USA		US Options
     '.BA'  => "ARS",    # Argentina	Buenos Aires
-    '.VI'  => "EUR",    # Austria		Vienna
+    '.VI'  => "EUR",    # Austria	Vienna
     '.AX'  => "AUD",    # Australia
-    '.SA'  => "BRL",    # Brazil		Sao Paolo
-    '.BR'  => "EUR",    # Belgium		Brussels
-    '.TO'  => "CAD",    # Canada		Toronto
-    '.V'   => "CAD",    # 		Toronto Venture
+    '.SA'  => "BRL",    # Brazil	Sao Paolo
+    '.BR'  => "EUR",    # Belgium	Brussels
+    '.TO'  => "CAD",    # Canada	Toronto
+    '.TRV' => "CAD",    # Canada        Toronto Venture
+    '.V'   => "CAD",    # Canada	Toronto Venture
     '.TRT' => "CAD",    # Canada        Toronto
     '.SN'  => "CLP",    # Chile		Santiago
     '.SS'  => "CNY",    # China		Shanghai
@@ -116,22 +120,30 @@ my %currencies_by_suffix = (
     '.SW'  => "CHF",    # Switzerland
 );
 
+our $DISPLAY    = 'AlphaVantage';
+our $FEATURES   = {'API_KEY' => 'registered user API key'};
+our @LABELS     = qw/date isodate open high low close volume last net p_change/;
+our $METHODHASH = {subroutine => \&alphavantage, 
+                   display => $DISPLAY, 
+                   labels => \@LABELS,
+                   features => $FEATURES};
 
-sub methods {
-    return ( alphavantage => \&alphavantage,
-             canada       => \&alphavantage,
-             usa          => \&alphavantage,
-             nyse         => \&alphavantage,
-             nasdaq       => \&alphavantage,
+sub labels {
+  my %m = methodinfo(); return map {$_ => [@{$m{$_}{labels}}] } keys %m;
+}
+
+sub methodinfo {
+    return ( 
+        alphavantage => $METHODHASH,
+        canada       => $METHODHASH,
+        usa          => $METHODHASH,
+        nyse         => $METHODHASH,
+        nasdaq       => $METHODHASH,
     );
 }
 
-{
-    my @labels = qw/date isodate open high low close volume last net p_change/;
-
-    sub labels {
-        return ( alphavantage => \@labels, );
-    }
+sub methods {
+  my %m = methodinfo(); return map {$_ => $m{$_}{subroutine} } keys %m;
 }
 
 sub sleep_before_query {
@@ -157,16 +169,36 @@ sub alphavantage {
 
     my @stocks = @_;
     my $quantity = @stocks;
-    my ( %info, $reply, $url, $code, $desc, $body );
+    my ( %info, $reply, $url, $code, $desc, $body, $ticker, $adjust );
     my $ua = $quoter->user_agent();
     my $launch_time = time();
 
+#   Since the JSON returned by the GLOBAL_QUOTE API does not specify
+#   the currency of the price data, there is no way to determine the
+#   correct currency without an additional call to the SYMBOL_SEARCH
+#   API. To avoid even slower throttling, this module expects the
+#   user to know which securties from certain countries may be traded
+#   in the non-ISO4217 currency.
+#   Example is LSE traded GBP.L and GBPG.L. GBP.L is traded in GBX,
+#   which is also known as GBp, and GBPG.L is traded in the iso-4217
+#   currency GBP (Great Britain Pounds).
+#   The user will add ".X" to symbols to return GBX priced securities
+#   as GBP.
+
     my $token = exists $quoter->{module_specific_data}->{alphavantage}->{API_KEY} ? 
                 $quoter->{module_specific_data}->{alphavantage}->{API_KEY}        :
                 $ENV{"ALPHAVANTAGE_API_KEY"};
 
     foreach my $stock (@stocks) {
 
+        if ($stock =~ /\.X$/) {
+            $adjust = 1;
+            ($ticker = $stock) =~ s/\.X$//;
+        } else {
+            $adjust = 0;
+            $ticker = $stock
+        }
+
         if ( !defined $token ) {
             $info{ $stock, 'success' } = 0;
             $info{ $stock, 'errormsg' } =
@@ -179,10 +211,10 @@ sub alphavantage {
             . '&apikey='
             . $token
             . '&symbol='
-            . $stock;
+            . $ticker;
 
         my $get_content = sub {
-            sleep_before_query();
+            # sleep_before_query();
             my $time=int(time()-$launch_time);
             # print STDERR "Query at:".$time."\n";
             $reply = $ua->request( GET $url);
@@ -218,17 +250,24 @@ sub alphavantage {
             $try_cnt += 1;
         }
 
-        if ( !$json_data || $json_data->{'Error Message'} ) {
+        if ( !$json_data ) {
             $info{ $stock, 'success' } = 0;
-            $info{ $stock, 'errormsg' } =
-                $json_data->{'Error Message'} || $json_data->{'Information'};
+            $info{ $stock, 'errormsg' } = 'Query returned no JSON';
+            next;
+        } elsif ( $json_data->{'Error Message'} ) {
+            $info{ $stock, 'success' } = 0;
+            $info{ $stock, 'errormsg' } = $json_data->{'Error Message'};
+            next;
+        } elsif ( $json_data->{'Information'} ) {
+            $info{ $stock, 'success' } = 0;
+            $info{ $stock, 'errormsg' } = $json_data->{'Information'};
             next;
         }
 
         my $quote = $json_data->{'Global Quote'};
-        if ( ! %{$quote} ) {
+        if ( ! $quote ) {
             $info{ $stock, 'success' } = 0;
-            $info{ $stock, 'errormsg' } = "json_data doesn't contain Global Quote";
+            $info{ $stock, 'errormsg' } = "json_data does not contain Global Quote";
             next;
         }
 
@@ -266,13 +305,13 @@ sub alphavantage {
         $quoter->store_date( \%info, $stock, { isodate => $quote->{'07. latest trading day'} } );
 
         # deduce currency
-        if ( $stock =~ /(\..*)/ ) {
+        if ( $ticker =~ /(\..*)/ ) {
             my $suffix = uc $1;
             if ( $currencies_by_suffix{$suffix} ) {
                 $info{ $stock, 'currency' } = $currencies_by_suffix{$suffix};
 
-                # divide GBP quotes by 100
-                if ( ($info{ $stock, 'currency' } eq 'GBP') || ($info{$stock,'currency'} eq 'GBX') ) {
+                # divide .X quotes by 100
+                if ( $adjust == 1 ) {
                     foreach my $field ( $quoter->default_currency_fields ) {
                         next unless ( $info{ $stock, $field } );
                         $info{ $stock, $field } =
@@ -305,7 +344,7 @@ sub alphavantage {
 
 =head1 NAME
 
-Finance::Quote::AlphaVantage - Obtain quotes from https://iexcloud.io
+Finance::Quote::AlphaVantage - Obtain quotes from https://alphavantage.co
 
 =head1 SYNOPSIS
 
@@ -313,7 +352,7 @@ Finance::Quote::AlphaVantage - Obtain quotes from https://iexcloud.io
     
     $q = Finance::Quote->new('AlphaVantage', alphavantage => {API_KEY => 'your-alphavantage-api-key'});
 
-    %info = Finance::Quote->fetch("IBM", "AAPL");
+    %info = $q->fetch('alphavantage', 'IBM', 'AAPL');
 
 =head1 DESCRIPTION
 
@@ -339,4 +378,22 @@ variable ALPHAVANTAGE_API_KEY.
 The following labels may be returned by Finance::Quote::AlphaVantage :
 symbol, open, close, high, low, last, volume, method, isodate, currency.
 
+=head1 CAVEATs
+
+Since the JSON returned by the GLOBAL_QUOTE API does not specify
+the currency of the price data, there is no way to determine the
+correct currency without an additional call to the SYMBOL_SEARCH
+API. To avoid even slower throttling, this module expects the
+user to know which securties from certain countries may be traded
+in the non-ISO4217 currency.
+
+An example are London Stock Exchange traded GBP.L (Global Petroleum Limited)
+and GBPG.L
+(Goldman Sachs Access UK Gilts 1-10 Years UCITS ETF CLASS GBP (Dist)).
+GBP.L is traded in GBX, which is also known as GBp (Great Britain Pence),
+and GBPG.L is traded in the iso-4217 currency GBP (Great Britain Pounds).
+The user will need to add ".X" to symbols to return GBX priced securities
+as GBP. For the example above the user would use the symbol GBP.L.X in
+the call to the alphavantage method for the prices to be output as GBP.
+
 =cut
